#!/usr/bin/python

import sys, os, re, subprocess, getopt

def usage():
    print 'Usage:', sys.argv[0], 
    print '''<OPTIONS> <keyfile>
 Anonymize all found IPv4 addresses found on the standard input
 using 'keyfile'.  Write anonymized version to standard output.

 Generate new keyfile with 
    scramble_ips NewKeyFile </dev/null
 
 Where <OPTIONS> can be:
    -h|--help: this help,
    -c|--cmd <path> : cmd to anonymize addresses (scramble_ips),
    --pass4=<bits>  : top bits of IPv4 addresses to pass unchanged,
    --pass6=<bits>  : top bits of IPv6 addresses to pass unchanged,
    -r|--reverse    : reverse anonymization.
    --astrisk       : flag changed IPs with an astrisk
    --mode=MODE     : set special output modes.  currently only "latex-save" is defined
'''

try:
    opts, args = getopt.gnu_getopt(sys.argv[1:], "hc:r", ["help", "cmd=", "pass4=", "pass6=", "reverse", "astrisk", "mode="])
except getopt.GetoptError, err:
    print str(err)
    usage()
    sys.exit(1)

o_pass4=0
o_pass6=0
o_reverse=""
o_astrisk = False
o_mode = ''
o_cmd='scramble_ips'

for o,ov in opts:
    if o in ("-h", "--help"):
        usage()
        sys.exit()
    elif o in ("-c", "--cmd"):
        o_cmd=ov
    elif o in ("--pass4"):
        o_pass4 = ov
    elif o in ("--pass6"):
        o_pass6 = ov
    elif o in ("--astrisk"):
        o_astrisk = True
    elif o in ("--mode"):
        o_mode = ov
    elif o in ("-r", "--reverse"):
        o_reverse = "-r"
    else:
        assert False, "unhandled option"

cmdopts=['--pass4='+str(o_pass4), '--pass6='+str(o_pass6)]

if o_reverse != "":
    cmdopts.append("-r")

if len(args) != 1:
    print >>sys.stderr, "Keyfile isn't specified"
    usage()
    sys.exit(1)

keyfile=args[0]
if not os.path.exists(keyfile):
    print >>sys.stderr, "Keyfile " + keyfile + " does not exist, trying to make up one..."
    os.system('</dev/null scramble_ips ' + keyfile)
    if not os.path.exists(keyfile):
        print >>sys.stderr, "Could not generate (scramble_ips is not in the path?)"
        sys.exit(1)
    else:
        print >>sys.stderr, "Generated keyfile: ", keyfile

ipv4re = re.compile('(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}') #re.compile('(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})')

#from http://people.spodhuis.org/phil.pennock/software/emit_ipv6_regexp-0.304
ipv6re = re.compile('(?:(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){6})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:::(?:(?:(?:[0-9a-fA-F]{1,4})):){5})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})))?::(?:(?:(?:[0-9a-fA-F]{1,4})):){4})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,1}(?:(?:[0-9a-fA-F]{1,4})))?::(?:(?:(?:[0-9a-fA-F]{1,4})):){3})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,2}(?:(?:[0-9a-fA-F]{1,4})))?::(?:(?:(?:[0-9a-fA-F]{1,4})):){2})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,3}(?:(?:[0-9a-fA-F]{1,4})))?::(?:(?:[0-9a-fA-F]{1,4})):)(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,4}(?:(?:[0-9a-fA-F]{1,4})))?::)(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,5}(?:(?:[0-9a-fA-F]{1,4})))?::)(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,6}(?:(?:[0-9a-fA-F]{1,4})))?::))))')

cmdlist = [o_cmd] + cmdopts + [keyfile]

try:
    pipe = subprocess.Popen(cmdlist, stdin=subprocess.PIPE, stdout=subprocess.PIPE, 
                            bufsize=1)
except OSError, e:
    print >>sys.stderr, "Failed to launch anon executable (" + " ".join(cmdlist) + ":", e
    print >>sys.stderr, "perhaps you need to do 'make install' in your dag_scrubber directory?"
    sys.exit(2)

anon_cache = {}
def anon(ipm):
    orig_ip = ipm.group()
    try:
        return anon_cache[orig_ip]
    except KeyError:
        pipe.stdin.write(orig_ip+'\n') 
        anon_ip = pipe.stdout.readline().rstrip()
        if o_astrisk:
            anon_ip += '*'
        if o_mode == 'latex-save':
            # yes this is a bit hacky
            anon_ip += ' % (' + orig_ip + ") \n  "
        anon_cache[orig_ip] = anon_ip
        return anon_ip


#change buffering of stdout
RBUFSZ=4096*1024
WBUFSZ=4096*1025
sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', WBUFSZ)   
while True:
    buf = sys.stdin.read(RBUFSZ)
    if not buf:
        break
    buf += sys.stdin.readline()
    print re.sub(ipv4re, anon, re.sub(ipv6re, anon, buf)),

